@@ -1,5 +1,5 @@
 /*
- * Copyright 2010-2019 The pygit2 contributors
+ * Copyright 2010-2021 The pygit2 contributors
  *
  * This file is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License, version 2,
@@ -27,7 +27,6 @@
 
 #define PY_SSIZE_T_CLEAN
 #include <Python.h>
-#include "blob.h"
 #include "diff.h"
 #include "error.h"
 #include "object.h"
@@ -145,7 +144,13 @@ static PyMethodDef Blob_methods[] = {
 };
 
 
-PyDoc_STRVAR(Blob_size__doc__, "Size.");
+PyDoc_STRVAR(Blob_size__doc__,
+    "Size in bytes.\n"
+    "\n"
+    "Example:\n"
+    "\n"
+    "    >>> print(blob.size)\n"
+    "    130\n");
 
 PyObject *
 Blob_size__get__(Blob *self)
@@ -169,8 +174,16 @@ Blob_is_binary__get__(Blob *self)
 
 
 PyDoc_STRVAR(Blob_data__doc__,
-  "The contents of the blob, a bytes string. This is the same as\n"
-  "Blob.read_raw()");
+    "The contents of the blob, a byte string. This is the same as\n"
+    "Blob.read_raw().\n"
+    "\n"
+    "Example, print the contents of the ``.gitignore`` file:\n"
+    "\n"
+    "    >>> blob = repo['d8022420bf6db02e906175f64f66676df539f2fd']\n"
+    "    >>> print(blob.data)\n"
+    "    MANIFEST\n"
+    "    build\n"
+    "    dist\n");
 
 PyGetSetDef Blob_getseters[] = {
     GETTER(Blob, size),
@@ -208,7 +221,7 @@ PyTypeObject BlobType = {
     0,                                         /* tp_getattr        */
     0,                                         /* tp_setattr        */
     0,                                         /* tp_compare        */
-    0,                                         /* tp_repr           */
+    (reprfunc)Object_repr,                     /* tp_repr           */
     0,                                         /* tp_as_number      */
     0,                                         /* tp_as_sequence    */
     0,                                         /* tp_as_mapping     */
