@@ -1,5 +1,5 @@
 /*
- * Copyright 2010-2019 The pygit2 contributors
+ * Copyright 2010-2021 The pygit2 contributors
  *
  * This file is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License, version 2,
@@ -247,15 +247,22 @@ DiffDelta_status_char(DiffDelta *self)
     return Py_BuildValue("C", status);
 }
 
-PyDoc_STRVAR(DiffDelta_is_binary__doc__, "True if binary data, False if not.");
+PyDoc_STRVAR(DiffDelta_is_binary__doc__,
+    "True if binary data, False if text, None if not (yet) known."
+);
 
 PyObject *
 DiffDelta_is_binary__get__(DiffDelta *self)
 {
-    if (!(self->flags & GIT_DIFF_FLAG_NOT_BINARY) &&
-            (self->flags & GIT_DIFF_FLAG_BINARY))
+    if (self->flags & GIT_DIFF_FLAG_BINARY)
         Py_RETURN_TRUE;
-    Py_RETURN_FALSE;
+
+    if (self->flags & GIT_DIFF_FLAG_NOT_BINARY)
+        Py_RETURN_FALSE;
+
+    // This means the file has not been loaded, so we don't know whether it's
+    // binary or text
+    Py_RETURN_NONE;
 }
 
 static void
@@ -571,6 +578,22 @@ Diff_len(Diff *self)
     return (Py_ssize_t)git_diff_num_deltas(self->diff);
 }
 
+PyDoc_STRVAR(Diff_patchid__doc__,
+    "Corresponding patchid.");
+
+PyObject *
+Diff_patchid__get__(Diff *self)
+{
+    git_oid oid;
+    int err;
+
+    err = git_diff_patchid(&oid, self->diff, NULL);
+    if (err < 0)
+        return Error_set(err);
+    return git_oid_to_python(&oid);
+}
+
+
 PyDoc_STRVAR(Diff_deltas__doc__, "Iterate over the diff deltas.");
 
 PyObject *
@@ -617,7 +640,7 @@ Diff_patch__get__(Diff *self)
         git_patch_free(patch);
     }
 
-    py_patch = to_unicode(buf.ptr, NULL, NULL);
+    py_patch = to_unicode_n(buf.ptr, buf.size, NULL, NULL);
     git_buf_dispose(&buf);
 
 cleanup:
@@ -815,7 +838,7 @@ DiffStats_format(DiffStats *self, PyObject *args, PyObject *kwds)
     if (err < 0)
         return Error_set(err);
 
-    str = to_unicode(buf.ptr, NULL, NULL);
+    str = to_unicode_n(buf.ptr, buf.size, NULL, NULL);
     git_buf_dispose(&buf);
 
     return str;
@@ -936,7 +959,15 @@ Diff_merge(Diff *self, PyObject *args)
 PyDoc_STRVAR(Diff_find_similar__doc__,
   "find_similar([flags, rename_threshold, copy_threshold, rename_from_rewrite_threshold, break_rewrite_threshold, rename_limit])\n"
   "\n"
-  "Find renamed files in diff and updates them in-place in the diff itself.");
+  "Transform a diff marking file renames, copies, etc.\n"
+  "\n"
+  "This modifies a diff in place, replacing old entries that look like\n"
+  "renames or copies with new entries reflecting those changes. This also "
+  "will, if requested, break modified files into add/remove pairs if the "
+  "amount of change is above a threshold.\n"
+  "\n"
+  "flags - Combination of GIT_DIFF_FIND_* and GIT_DIFF_BREAK_* constants."
+  );
 
 PyObject *
 Diff_find_similar(Diff *self, PyObject *args, PyObject *kwds)
@@ -944,10 +975,14 @@ Diff_find_similar(Diff *self, PyObject *args, PyObject *kwds)
     int err;
     git_diff_find_options opts = GIT_DIFF_FIND_OPTIONS_INIT;
 
-    char *keywords[] = {"flags", "rename_threshold", "copy_threshold", "rename_from_rewrite_threshold", "break_rewrite_threshold", "rename_limit", NULL};
+    char *keywords[] = {"flags", "rename_threshold", "copy_threshold",
+                        "rename_from_rewrite_threshold",
+                        "break_rewrite_threshold", "rename_limit", NULL};
 
     if (!PyArg_ParseTupleAndKeywords(args, kwds, "|iHHHHI", keywords,
-                &opts.flags, &opts.rename_threshold, &opts.copy_threshold, &opts.rename_from_rewrite_threshold, &opts.break_rewrite_threshold, &opts.rename_limit))
+            &opts.flags, &opts.rename_threshold, &opts.copy_threshold,
+            &opts.rename_from_rewrite_threshold, &opts.break_rewrite_threshold,
+            &opts.rename_limit))
         return NULL;
 
     err = git_diff_find_similar(self->diff, &opts);
@@ -1002,16 +1037,12 @@ Diff_parse_diff(PyObject *self, PyObject *py_str)
 {
     /* A wrapper around git_diff_from_buffer */
     git_diff *diff;
-    const char *content = NULL;
-    int err;
-    PyObject *tvalue;
 
-    content = py_str_borrow_c_str(&tvalue, py_str, NULL);
+    const char *content = pgit_borrow(py_str);
     if (content == NULL)
         return NULL;
 
-    err = git_diff_from_buffer(&diff, content, strlen(content));
-    Py_DECREF(tvalue);
+    int err = git_diff_from_buffer(&diff, content, strlen(content));
     if (err < 0)
         return Error_set(err);
 
@@ -1030,6 +1061,7 @@ PyGetSetDef Diff_getsetters[] = {
     GETTER(Diff, deltas),
     GETTER(Diff, patch),
     GETTER(Diff, stats),
+    GETTER(Diff, patchid),
     {NULL}
 };
 
