@@ -1,5 +1,5 @@
 /*
- * Copyright 2010-2019 The pygit2 contributors
+ * Copyright 2010-2021 The pygit2 contributors
  *
  * This file is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License, version 2,
@@ -102,7 +102,7 @@ Object_oid__get__(Object *self)
 
 PyDoc_STRVAR(Object_hex__doc__,
     "Hexadecimal representation of the object id. This is a shortcut for\n"
-    "Object.oid.hex\n"
+    "Object.id.hex\n"
     "This attribute is deprecated, please use 'id'\n");
 
 PyObject *
@@ -175,6 +175,9 @@ PyDoc_STRVAR(Object_raw_name__doc__, "Name (bytes).");
 PyObject *
 Object_raw_name__get__(Object *self)
 {
+    if (self->entry == NULL)
+        Py_RETURN_NONE;
+
     return PyBytes_FromString(git_tree_entry_name(self->entry));
 }
 
@@ -255,6 +258,20 @@ Object_hash(Object *self)
     return ret;
 }
 
+PyObject *
+Object_repr(Object *self)
+{
+    char hex[GIT_OID_HEXSZ + 1];
+
+    git_oid_fmt(hex, Object__id(self));
+    hex[GIT_OID_HEXSZ] = '\0';
+
+    return PyUnicode_FromFormat("<pygit2.Object{%s:%s}>",
+        git_object_type2string(Object__type(self)),
+        hex
+    );
+}
+
 PyObject *
 Object_richcompare(PyObject *o1, PyObject *o2, int op)
 {
@@ -322,7 +339,7 @@ PyTypeObject ObjectType = {
     0,                                         /* tp_getattr        */
     0,                                         /* tp_setattr        */
     0,                                         /* tp_compare        */
-    0,                                         /* tp_repr           */
+    (reprfunc)Object_repr,                     /* tp_repr           */
     0,                                         /* tp_as_number      */
     0,                                         /* tp_as_sequence    */
     0,                                         /* tp_as_mapping     */
