@@ -1,5 +1,5 @@
 /*
- * Copyright 2010-2019 The pygit2 contributors
+ * Copyright 2010-2021 The pygit2 contributors
  *
  * This file is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License, version 2,
@@ -36,10 +36,8 @@
 
 git_object* Object__load(Object *self);
 const git_oid* Object__id(Object *self);
-PyObject* Object_get_oid(Object *self);
-PyObject* Object_get_hex(Object *self);
-PyObject* Object_get_type(Object *self);
 PyObject* Object_read_raw(Object *self);
+PyObject* Object_repr(Object *self);
 PyObject* wrap_object(git_object *c_object, Repository *repo, const git_tree_entry *entry);
 
 #endif
