@@ -1,5 +1,5 @@
 /*
- * Copyright 2010-2019 The pygit2 contributors
+ * Copyright 2010-2021 The pygit2 contributors
  *
  * This file is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License, version 2,
@@ -36,9 +36,9 @@
 #include "oid.h"
 #include "options.h"
 
-extern PyObject *GitError;
-extern PyObject *AlreadyExistsError;
-extern PyObject *InvalidSpecError;
+PyObject *GitError;
+PyObject *AlreadyExistsError;
+PyObject *InvalidSpecError;
 
 extern PyTypeObject RepositoryType;
 extern PyTypeObject OdbType;
@@ -63,7 +63,11 @@ extern PyTypeObject TreeIterType;
 extern PyTypeObject BlobType;
 extern PyTypeObject TagType;
 extern PyTypeObject WalkerType;
+extern PyTypeObject RefdbType;
+extern PyTypeObject RefdbBackendType;
+extern PyTypeObject RefdbFsBackendType;
 extern PyTypeObject ReferenceType;
+extern PyTypeObject RevSpecType;
 extern PyTypeObject RefLogIterType;
 extern PyTypeObject RefLogEntryType;
 extern PyTypeObject BranchType;
@@ -74,6 +78,7 @@ extern PyTypeObject NoteType;
 extern PyTypeObject NoteIterType;
 extern PyTypeObject WorktreeType;
 extern PyTypeObject MailmapType;
+extern PyTypeObject StashType;
 
 
 PyDoc_STRVAR(discover_repository__doc__,
@@ -85,17 +90,29 @@ PyObject *
 discover_repository(PyObject *self, PyObject *args)
 {
     git_buf repo_path = {NULL};
-    const char *path;
-    PyObject *py_repo_path;
+    const char *path = NULL;
+    PyBytesObject *py_path = NULL;
     int across_fs = 0;
+    PyBytesObject *py_ceiling_dirs = NULL;
     const char *ceiling_dirs = NULL;
+    PyObject *py_repo_path = NULL;
     int err;
 
-    if (!PyArg_ParseTuple(args, "s|Is", &path, &across_fs, &ceiling_dirs))
+    if (!PyArg_ParseTuple(args, "O&|IO&", PyUnicode_FSConverter, &py_path, &across_fs,
+                          PyUnicode_FSConverter, &py_ceiling_dirs))
         return NULL;
 
+    if (py_path != NULL)
+        path = PyBytes_AS_STRING(py_path);
+    if (py_ceiling_dirs != NULL)
+        ceiling_dirs = PyBytes_AS_STRING(py_ceiling_dirs);
+
     memset(&repo_path, 0, sizeof(git_buf));
     err = git_repository_discover(&repo_path, path, across_fs, ceiling_dirs);
+
+    Py_XDECREF(py_path);
+    Py_XDECREF(py_ceiling_dirs);
+
     if (err == GIT_ENOTFOUND)
         Py_RETURN_NONE;
     if (err < 0)
@@ -116,13 +133,18 @@ PyObject *
 hashfile(PyObject *self, PyObject *args)
 {
     git_oid oid;
-    const char* path;
+    PyBytesObject *py_path = NULL;
+    const char* path = NULL;
     int err;
 
-    if (!PyArg_ParseTuple(args, "s", &path))
+    if (!PyArg_ParseTuple(args, "O&", PyUnicode_FSConverter, &py_path))
         return NULL;
 
+    if (py_path != NULL)
+        path = PyBytes_AS_STRING(py_path);
+
     err = git_odb_hashfile(&oid, path, GIT_OBJ_BLOB);
+    Py_XDECREF(py_path);
     if (err < 0)
         return Error_set(err);
 
@@ -155,20 +177,27 @@ hash(PyObject *self, PyObject *args)
 
 
 PyDoc_STRVAR(init_file_backend__doc__,
-  "init_file_backend(path) -> object\n"
+  "init_file_backend(path[, flags]) -> object\n"
   "\n"
-  "open repo backend given path.");
+  "Open repo backend given path.");
 PyObject *
 init_file_backend(PyObject *self, PyObject *args)
 {
+    PyBytesObject *py_path = NULL;
     const char* path = NULL;
+    unsigned int flags = 0;
     int err = GIT_OK;
     git_repository *repository = NULL;
-    if (!PyArg_ParseTuple(args, "s", &path)) {
-        return NULL;
-    }
 
-    err = git_repository_open(&repository, path);
+    if (!PyArg_ParseTuple(args, "O&|I", PyUnicode_FSConverter, &py_path, &flags))
+        return NULL;
+    if (py_path != NULL)
+        path = PyBytes_AS_STRING(py_path);
+
+    err = git_repository_open_ext(&repository, path, flags, NULL);
+
+    Py_XDECREF(py_path);
+
     if (err < 0) {
         Error_set_str(err, path);
         goto cleanup;
@@ -196,16 +225,11 @@ PyDoc_STRVAR(reference_is_valid_name__doc__,
 PyObject *
 reference_is_valid_name(PyObject *self, PyObject *py_refname)
 {
-    char* refname;
-    int result;
-
-    refname = py_str_to_c_str(py_refname, NULL);
-    if (refname == NULL) {
+    const char *refname = pgit_borrow(py_refname);
+    if (refname == NULL)
         return NULL;
-    }
 
-    result = git_reference_is_valid_name(refname);
-    free(refname);
+    int result = git_reference_is_valid_name(refname);
     return PyBool_FromLong(result);
 }
 
@@ -250,9 +274,23 @@ PyMethodDef module_methods[] = {
     {NULL}
 };
 
-PyObject *
-moduleinit(PyObject* m)
+
+struct PyModuleDef moduledef = {
+    PyModuleDef_HEAD_INIT,
+    "_pygit2",                       /* m_name */
+    "Python bindings for libgit2.",  /* m_doc */
+    -1,                              /* m_size */
+    module_methods,                  /* m_methods */
+    NULL,                            /* m_reload */
+    NULL,                            /* m_traverse */
+    NULL,                            /* m_clear */
+    NULL,                            /* m_free */
+};
+
+PyMODINIT_FUNC
+PyInit__pygit2(void)
 {
+    PyObject *m = PyModule_Create(&moduledef);
     if (m == NULL)
         return NULL;
 
@@ -290,19 +328,12 @@ moduleinit(PyObject* m)
     ADD_CONSTANT_INT(m, GIT_OPT_ENABLE_UNSAVED_INDEX_SAFETY);
     ADD_CONSTANT_INT(m, GIT_OPT_GET_PACK_MAX_OBJECTS);
     ADD_CONSTANT_INT(m, GIT_OPT_SET_PACK_MAX_OBJECTS);
+    ADD_CONSTANT_INT(m, GIT_OPT_DISABLE_PACK_KEEP_FILE_CHECKS);
 
-    /* Errors */
-    GitError = PyErr_NewException("_pygit2.GitError", NULL, NULL);
-    Py_INCREF(GitError);
-    PyModule_AddObject(m, "GitError", GitError);
-
-    AlreadyExistsError = PyErr_NewException("_pygit2.AlreadyExistsError", PyExc_ValueError, NULL);
-    Py_INCREF(AlreadyExistsError);
-    PyModule_AddObject(m, "AlreadyExistsError", AlreadyExistsError);
-
-    InvalidSpecError = PyErr_NewException("_pygit2.InvalidSpecError", PyExc_ValueError, NULL);
-    Py_INCREF(InvalidSpecError);
-    PyModule_AddObject(m, "InvalidSpecError", InvalidSpecError);
+    /* Exceptions */
+    ADD_EXC(m, GitError, NULL);
+    ADD_EXC(m, AlreadyExistsError, PyExc_ValueError);
+    ADD_EXC(m, InvalidSpecError, PyExc_ValueError);
 
     /* Repository */
     INIT_TYPE(RepositoryType, NULL, PyType_GenericNew)
@@ -312,7 +343,7 @@ moduleinit(PyObject* m)
     INIT_TYPE(OdbType, NULL, PyType_GenericNew)
     ADD_TYPE(m, Odb)
 
-    INIT_TYPE(OdbBackendType, NULL, NULL)
+    INIT_TYPE(OdbBackendType, NULL, PyType_GenericNew)
     ADD_TYPE(m, OdbBackend)
     INIT_TYPE(OdbBackendPackType, &OdbBackendType, PyType_GenericNew)
     ADD_TYPE(m, OdbBackendPack)
@@ -374,10 +405,19 @@ moduleinit(PyObject* m)
     ADD_CONSTANT_INT(m, GIT_RESET_MIXED)
     ADD_CONSTANT_INT(m, GIT_RESET_HARD)
 
+    /* Refdb */
+    INIT_TYPE(RefdbType, NULL, PyType_GenericNew)
+    ADD_TYPE(m, Refdb)
+
+    INIT_TYPE(RefdbBackendType, NULL, PyType_GenericNew)
+    ADD_TYPE(m, RefdbBackend)
+    INIT_TYPE(RefdbFsBackendType, &RefdbBackendType, PyType_GenericNew)
+    ADD_TYPE(m, RefdbFsBackend)
+
     /*
      * References
      */
-    INIT_TYPE(ReferenceType, NULL, NULL)
+    INIT_TYPE(ReferenceType, NULL, PyType_GenericNew)
     INIT_TYPE(RefLogEntryType, NULL, NULL)
     INIT_TYPE(RefLogIterType, NULL, NULL)
     INIT_TYPE(NoteType, NULL, NULL)
@@ -390,6 +430,15 @@ moduleinit(PyObject* m)
     ADD_CONSTANT_INT(m, GIT_REF_SYMBOLIC)
     ADD_CONSTANT_INT(m, GIT_REF_LISTALL)
 
+    /*
+     * RevSpec
+     */
+    INIT_TYPE(RevSpecType, NULL, NULL)
+    ADD_TYPE(m, RevSpec)
+    ADD_CONSTANT_INT(m, GIT_REVPARSE_SINGLE)
+    ADD_CONSTANT_INT(m, GIT_REVPARSE_RANGE)
+    ADD_CONSTANT_INT(m, GIT_REVPARSE_MERGE_BASE)
+
     /*
      * Worktree
      */
@@ -435,6 +484,7 @@ moduleinit(PyObject* m)
     ADD_CONSTANT_INT(m, GIT_CHECKOUT_DONT_UPDATE_INDEX)
     ADD_CONSTANT_INT(m, GIT_CHECKOUT_NO_REFRESH)
     ADD_CONSTANT_INT(m, GIT_CHECKOUT_DISABLE_PATHSPEC_MATCH)
+    ADD_CONSTANT_INT(m, GIT_CHECKOUT_SKIP_LOCKED_DIRECTORIES)
 
     /*
      * Diff
@@ -455,51 +505,71 @@ moduleinit(PyObject* m)
     ADD_TYPE(m, DiffLine)
     ADD_TYPE(m, DiffStats)
     ADD_TYPE(m, Patch)
+
+    /* (git_diff_options in libgit2) */
     ADD_CONSTANT_INT(m, GIT_DIFF_NORMAL)
     ADD_CONSTANT_INT(m, GIT_DIFF_REVERSE)
+    ADD_CONSTANT_INT(m, GIT_DIFF_INCLUDE_IGNORED)
+    ADD_CONSTANT_INT(m, GIT_DIFF_RECURSE_IGNORED_DIRS)
+    ADD_CONSTANT_INT(m, GIT_DIFF_INCLUDE_UNTRACKED)
+    ADD_CONSTANT_INT(m, GIT_DIFF_RECURSE_UNTRACKED_DIRS)
+    ADD_CONSTANT_INT(m, GIT_DIFF_INCLUDE_UNMODIFIED)
+    ADD_CONSTANT_INT(m, GIT_DIFF_INCLUDE_TYPECHANGE)
+    ADD_CONSTANT_INT(m, GIT_DIFF_INCLUDE_TYPECHANGE_TREES)
+    ADD_CONSTANT_INT(m, GIT_DIFF_IGNORE_FILEMODE)
+    ADD_CONSTANT_INT(m, GIT_DIFF_IGNORE_SUBMODULES)
+    ADD_CONSTANT_INT(m, GIT_DIFF_IGNORE_CASE)
+    ADD_CONSTANT_INT(m, GIT_DIFF_INCLUDE_CASECHANGE)
+    ADD_CONSTANT_INT(m, GIT_DIFF_DISABLE_PATHSPEC_MATCH)
+    ADD_CONSTANT_INT(m, GIT_DIFF_SKIP_BINARY_CHECK)
+    ADD_CONSTANT_INT(m, GIT_DIFF_ENABLE_FAST_UNTRACKED_DIRS)
+    ADD_CONSTANT_INT(m, GIT_DIFF_UPDATE_INDEX)
+    ADD_CONSTANT_INT(m, GIT_DIFF_INCLUDE_UNREADABLE)
+    ADD_CONSTANT_INT(m, GIT_DIFF_INCLUDE_UNREADABLE_AS_UNTRACKED)
+    ADD_CONSTANT_INT(m, GIT_DIFF_INDENT_HEURISTIC)
     ADD_CONSTANT_INT(m, GIT_DIFF_FORCE_TEXT)
+    ADD_CONSTANT_INT(m, GIT_DIFF_FORCE_BINARY)
     ADD_CONSTANT_INT(m, GIT_DIFF_IGNORE_WHITESPACE)
     ADD_CONSTANT_INT(m, GIT_DIFF_IGNORE_WHITESPACE_CHANGE)
     ADD_CONSTANT_INT(m, GIT_DIFF_IGNORE_WHITESPACE_EOL)
-    ADD_CONSTANT_INT(m, GIT_DIFF_IGNORE_SUBMODULES)
+    ADD_CONSTANT_INT(m, GIT_DIFF_SHOW_UNTRACKED_CONTENT)
+    ADD_CONSTANT_INT(m, GIT_DIFF_SHOW_UNMODIFIED)
     ADD_CONSTANT_INT(m, GIT_DIFF_PATIENCE)
     ADD_CONSTANT_INT(m, GIT_DIFF_MINIMAL)
-    ADD_CONSTANT_INT(m, GIT_DIFF_INCLUDE_IGNORED)
-    ADD_CONSTANT_INT(m, GIT_DIFF_INCLUDE_UNTRACKED)
-    ADD_CONSTANT_INT(m, GIT_DIFF_INCLUDE_UNMODIFIED)
-    ADD_CONSTANT_INT(m, GIT_DIFF_RECURSE_UNTRACKED_DIRS)
-    ADD_CONSTANT_INT(m, GIT_DIFF_RECURSE_UNTRACKED_DIRS)
-    ADD_CONSTANT_INT(m, GIT_DIFF_DISABLE_PATHSPEC_MATCH)
-    ADD_CONSTANT_INT(m, GIT_DIFF_IGNORE_CASE)
-    ADD_CONSTANT_INT(m, GIT_DIFF_SHOW_UNTRACKED_CONTENT)
-    ADD_CONSTANT_INT(m, GIT_DIFF_SKIP_BINARY_CHECK)
     ADD_CONSTANT_INT(m, GIT_DIFF_SHOW_BINARY)
-    ADD_CONSTANT_INT(m, GIT_DIFF_INCLUDE_TYPECHANGE)
-    ADD_CONSTANT_INT(m, GIT_DIFF_INCLUDE_TYPECHANGE_TREES)
-    ADD_CONSTANT_INT(m, GIT_DIFF_RECURSE_IGNORED_DIRS)
+
+    /* Formatting options for diff stats (git_diff_stats_format_t in libgit2) */
     ADD_CONSTANT_INT(m, GIT_DIFF_STATS_NONE)
     ADD_CONSTANT_INT(m, GIT_DIFF_STATS_FULL)
     ADD_CONSTANT_INT(m, GIT_DIFF_STATS_SHORT)
     ADD_CONSTANT_INT(m, GIT_DIFF_STATS_NUMBER)
     ADD_CONSTANT_INT(m, GIT_DIFF_STATS_INCLUDE_SUMMARY)
-    /* Flags for diff find similar */
-    /* --find-renames */
-    ADD_CONSTANT_INT(m, GIT_DIFF_FIND_RENAMES)
-    /* --break-rewrites=N */
-    ADD_CONSTANT_INT(m, GIT_DIFF_FIND_RENAMES_FROM_REWRITES)
-    /* --find-copies */
-    ADD_CONSTANT_INT(m, GIT_DIFF_FIND_COPIES)
-    /* --find-copies-harder */
-    ADD_CONSTANT_INT(m, GIT_DIFF_FIND_COPIES_FROM_UNMODIFIED)
-    /* --break-rewrites=/M */
-    ADD_CONSTANT_INT(m, GIT_DIFF_FIND_AND_BREAK_REWRITES)
 
-    /* DiffDelta and DiffFile flags */
+    /* Flags for Diff.find_similar (git_diff_find_t in libgit2) */
+    ADD_CONSTANT_INT(m, GIT_DIFF_FIND_BY_CONFIG) /** Obey diff.renames */
+    ADD_CONSTANT_INT(m, GIT_DIFF_FIND_RENAMES) /* --find-renames */
+    ADD_CONSTANT_INT(m, GIT_DIFF_FIND_RENAMES_FROM_REWRITES) /* --break-rewrites=N */
+    ADD_CONSTANT_INT(m, GIT_DIFF_FIND_COPIES) /* --find-copies */
+    ADD_CONSTANT_INT(m, GIT_DIFF_FIND_COPIES_FROM_UNMODIFIED) /* --find-copies-harder */
+    ADD_CONSTANT_INT(m, GIT_DIFF_FIND_REWRITES) /* --break-rewrites=/M */
+    ADD_CONSTANT_INT(m, GIT_DIFF_BREAK_REWRITES)
+    ADD_CONSTANT_INT(m, GIT_DIFF_FIND_AND_BREAK_REWRITES)
+    ADD_CONSTANT_INT(m, GIT_DIFF_FIND_FOR_UNTRACKED)
+    ADD_CONSTANT_INT(m, GIT_DIFF_FIND_ALL) /* Turn on all finding features. */
+    ADD_CONSTANT_INT(m, GIT_DIFF_FIND_IGNORE_LEADING_WHITESPACE)
+    ADD_CONSTANT_INT(m, GIT_DIFF_FIND_IGNORE_WHITESPACE)
+    ADD_CONSTANT_INT(m, GIT_DIFF_FIND_DONT_IGNORE_WHITESPACE)
+    ADD_CONSTANT_INT(m, GIT_DIFF_FIND_EXACT_MATCH_ONLY)
+    ADD_CONSTANT_INT(m, GIT_DIFF_BREAK_REWRITES_FOR_RENAMES_ONLY)
+    ADD_CONSTANT_INT(m, GIT_DIFF_FIND_REMOVE_UNMODIFIED)
+
+    /* DiffDelta and DiffFile flags (git_diff_flag_t in libgit2) */
     ADD_CONSTANT_INT(m, GIT_DIFF_FLAG_BINARY)
     ADD_CONSTANT_INT(m, GIT_DIFF_FLAG_NOT_BINARY)
     ADD_CONSTANT_INT(m, GIT_DIFF_FLAG_VALID_ID)
+    ADD_CONSTANT_INT(m, GIT_DIFF_FLAG_EXISTS)
 
-    /* DiffDelta.status */
+    /* DiffDelta.status (git_delta_t in libgit2) */
     ADD_CONSTANT_INT(m, GIT_DELTA_UNMODIFIED)
     ADD_CONSTANT_INT(m, GIT_DELTA_ADDED)
     ADD_CONSTANT_INT(m, GIT_DELTA_DELETED)
@@ -510,6 +580,7 @@ moduleinit(PyObject* m)
     ADD_CONSTANT_INT(m, GIT_DELTA_UNTRACKED)
     ADD_CONSTANT_INT(m, GIT_DELTA_TYPECHANGE)
     ADD_CONSTANT_INT(m, GIT_DELTA_UNREADABLE)
+    ADD_CONSTANT_INT(m, GIT_DELTA_CONFLICTED)
 
     /* Config */
     ADD_CONSTANT_INT(m, GIT_CONFIG_LEVEL_LOCAL);
@@ -523,6 +594,9 @@ moduleinit(PyObject* m)
     ADD_CONSTANT_INT(m, GIT_BLAME_TRACK_COPIES_SAME_COMMIT_MOVES)
     ADD_CONSTANT_INT(m, GIT_BLAME_TRACK_COPIES_SAME_COMMIT_COPIES)
     ADD_CONSTANT_INT(m, GIT_BLAME_TRACK_COPIES_ANY_COMMIT_COPIES)
+    ADD_CONSTANT_INT(m, GIT_BLAME_FIRST_PARENT)
+    ADD_CONSTANT_INT(m, GIT_BLAME_USE_MAILMAP)
+    ADD_CONSTANT_INT(m, GIT_BLAME_IGNORE_WHITESPACE)
 
     /* Merge */
     ADD_CONSTANT_INT(m, GIT_MERGE_ANALYSIS_NONE)
@@ -530,6 +604,9 @@ moduleinit(PyObject* m)
     ADD_CONSTANT_INT(m, GIT_MERGE_ANALYSIS_UP_TO_DATE)
     ADD_CONSTANT_INT(m, GIT_MERGE_ANALYSIS_FASTFORWARD)
     ADD_CONSTANT_INT(m, GIT_MERGE_ANALYSIS_UNBORN)
+    ADD_CONSTANT_INT(m, GIT_MERGE_PREFERENCE_NONE)
+    ADD_CONSTANT_INT(m, GIT_MERGE_PREFERENCE_NO_FASTFORWARD)
+    ADD_CONSTANT_INT(m, GIT_MERGE_PREFERENCE_FASTFORWARD_ONLY)
 
     /* Describe */
     ADD_CONSTANT_INT(m, GIT_DESCRIBE_DEFAULT);
@@ -544,33 +621,25 @@ moduleinit(PyObject* m)
     ADD_CONSTANT_INT(m, GIT_STASH_APPLY_DEFAULT);
     ADD_CONSTANT_INT(m, GIT_STASH_APPLY_REINSTATE_INDEX);
 
+    /* Apply location */
+    ADD_CONSTANT_INT(m, GIT_APPLY_LOCATION_WORKDIR);
+    ADD_CONSTANT_INT(m, GIT_APPLY_LOCATION_INDEX);
+    ADD_CONSTANT_INT(m, GIT_APPLY_LOCATION_BOTH);
+
     /* Mailmap */
     INIT_TYPE(MailmapType, NULL, PyType_GenericNew)
     ADD_TYPE(m, Mailmap)
 
+    /* Stash */
+    INIT_TYPE(StashType, NULL, NULL)
+    ADD_TYPE(m, Stash)
+
     /* Global initialization of libgit2 */
     git_libgit2_init();
 
     return m;
-}
 
-
-struct PyModuleDef moduledef = {
-    PyModuleDef_HEAD_INIT,
-    "_pygit2",                       /* m_name */
-    "Python bindings for libgit2.",  /* m_doc */
-    -1,                              /* m_size */
-    module_methods,                  /* m_methods */
-    NULL,                            /* m_reload */
-    NULL,                            /* m_traverse */
-    NULL,                            /* m_clear */
-    NULL,                            /* m_free */
-};
-
-PyMODINIT_FUNC
-PyInit__pygit2(void)
-{
-    PyObject* m;
-    m = PyModule_Create(&moduledef);
-    return moduleinit(m);
+fail:
+    Py_DECREF(m);
+    return NULL;
 }
