@@ -1,5 +1,5 @@
 /*
- * Copyright 2010-2019 The pygit2 contributors
+ * Copyright 2010-2021 The pygit2 contributors
  *
  * This file is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License, version 2,
@@ -30,11 +30,13 @@
 #include "error.h"
 #include "types.h"
 #include "reference.h"
+#include "revspec.h"
 #include "utils.h"
 #include "odb.h"
 #include "object.h"
 #include "oid.h"
 #include "note.h"
+#include "refdb.h"
 #include "repository.h"
 #include "diff.h"
 #include "branch.h"
@@ -56,8 +58,10 @@ extern PyTypeObject TreeBuilderType;
 extern PyTypeObject ConfigType;
 extern PyTypeObject DiffType;
 extern PyTypeObject ReferenceType;
+extern PyTypeObject RevSpecType;
 extern PyTypeObject NoteType;
 extern PyTypeObject NoteIterType;
+extern PyTypeObject StashType;
 
 /* forward-declaration for Repsository._from_c() */
 PyTypeObject RepositoryType;
@@ -80,7 +84,7 @@ wrap_repository(git_repository *c_repo)
 int
 Repository_init(Repository *self, PyObject *args, PyObject *kwds)
 {
-    PyObject *backend;
+    PyObject *backend = NULL;
 
     if (kwds && PyDict_Size(kwds) > 0) {
         PyErr_SetString(PyExc_TypeError,
@@ -88,10 +92,23 @@ Repository_init(Repository *self, PyObject *args, PyObject *kwds)
         return -1;
     }
 
-    if (!PyArg_ParseTuple(args, "O", &backend)) {
+    if (!PyArg_ParseTuple(args, "|O", &backend)) {
         return -1;
     }
 
+    if (backend == NULL) {
+        /* Create repository without odb/refdb */
+        int err = git_repository_new(&self->repo);
+        if (err != 0) {
+            Error_set(err);
+            return -1;
+        }
+        self->owned = 1;
+        self->config = NULL;
+        self->index = NULL;
+        return 0;
+    }
+
     self->repo = PyCapsule_GetPointer(backend, "backend");
     if (self->repo == NULL) {
         PyErr_SetString(PyExc_TypeError,
@@ -248,6 +265,19 @@ Repository_is_bare__get__(Repository *self)
 }
 
 
+PyDoc_STRVAR(Repository_is_shallow__doc__,
+  "Check if a repository is a shallow repository.");
+
+PyObject *
+Repository_is_shallow__get__(Repository *self)
+{
+    if (git_repository_is_shallow(self->repo) > 0)
+        Py_RETURN_TRUE;
+
+    Py_RETURN_FALSE;
+}
+
+
 PyDoc_STRVAR(Repository_git_object_lookup_prefix__doc__,
   "git_object_lookup_prefix(oid) -> Object\n"
   "\n"
@@ -288,10 +318,11 @@ Repository_lookup_branch(Repository *self, PyObject *args)
 {
     git_reference *c_reference;
     const char *c_name;
+    Py_ssize_t c_name_len;
     git_branch_t branch_type = GIT_BRANCH_LOCAL;
     int err;
 
-    if (!PyArg_ParseTuple(args, "s|I", &c_name, &branch_type))
+    if (!PyArg_ParseTuple(args, "s#|I", &c_name, &c_name_len, &branch_type))
         return NULL;
 
     err = git_branch_lookup(&c_reference, self->repo, c_name, branch_type);
@@ -335,40 +366,99 @@ PyDoc_STRVAR(Repository_revparse_single__doc__,
 PyObject *
 Repository_revparse_single(Repository *self, PyObject *py_spec)
 {
-    git_object *c_obj;
-    const char *c_spec;
-    PyObject *tspec;
-    int err;
-
-    /* 1- Get the C revision spec */
-    c_spec = py_str_borrow_c_str(&tspec, py_spec, NULL);
+    /* Get the C revision spec */
+    const char *c_spec = pgit_borrow(py_spec);
     if (c_spec == NULL)
         return NULL;
 
-    /* 2- Lookup */
-    err = git_revparse_single(&c_obj, self->repo, c_spec);
-    if (err < 0) {
-        PyObject *err_obj = Error_set_str(err, c_spec);
-        Py_DECREF(tspec);
-        return err_obj;
-    }
-
-    Py_DECREF(tspec);
+    /* Lookup */
+    git_object *c_obj;
+    int err = git_revparse_single(&c_obj, self->repo, c_spec);
+    if (err)
+        return Error_set_str(err, c_spec);
 
     return wrap_object(c_obj, self, NULL);
 }
 
 
+PyDoc_STRVAR(Repository_revparse_ext__doc__,
+  "revparse_ext(revision) -> (Object, Reference)\n"
+  "\n"
+  "Find a single object and intermediate reference, as specified by a revision\n"
+  "string. See `man gitrevisions`, or the documentation for `git rev-parse`\n"
+  "for information on the syntax accepted.\n"
+  "\n"
+  "In some cases (@{<-n>} or <branchname>@{upstream}), the expression may\n"
+  "point to an intermediate reference, which is returned in the second element\n"
+  "of the result tuple.");
+
+PyObject *
+Repository_revparse_ext(Repository *self, PyObject *py_spec)
+{
+    /* Get the C revision spec */
+    const char *c_spec = pgit_borrow(py_spec);
+    if (c_spec == NULL)
+        return NULL;
+
+    /* Lookup */
+    git_object *c_obj = NULL;
+    git_reference *c_ref = NULL;
+    int err = git_revparse_ext(&c_obj, &c_ref, self->repo, c_spec);
+    if (err)
+        return Error_set_str(err, c_spec);
+
+    PyObject *py_obj = wrap_object(c_obj, self, NULL);
+    PyObject *py_ref = NULL;
+    if (c_ref != NULL) {
+        py_ref = wrap_reference(c_ref, self);
+    } else {
+        py_ref = Py_None;
+        Py_INCREF(Py_None);
+    }
+    return Py_BuildValue("NN", py_obj, py_ref);
+}
+
+
+PyDoc_STRVAR(Repository_revparse__doc__,
+  "revparse(revspec) -> RevSpec\n"
+  "\n"
+  "Parse a revision string for from, to, and intent. See `man gitrevisions`,\n"
+  "or the documentation for `git rev-parse` for information on the syntax\n"
+  "accepted.");
+
+PyObject *
+Repository_revparse(Repository *self, PyObject *py_spec)
+{
+    /* Get the C revision spec */
+    const char *c_spec = pgit_borrow(py_spec);
+    if (c_spec == NULL)
+        return NULL;
+
+    /* Lookup */
+    git_revspec revspec;
+    int err = git_revparse(&revspec, self->repo, c_spec);
+    if (err) {
+        return Error_set_str(err, c_spec);
+    }
+    return wrap_revspec(&revspec, self);
+}
+
+
 PyDoc_STRVAR(Repository_path__doc__,
   "The normalized path to the git repository.");
 
 PyObject *
 Repository_path__get__(Repository *self, void *closure)
 {
+    const char *c_path;
     if (self->repo == NULL)
       Py_RETURN_NONE;
 
-    return to_path(git_repository_path(self->repo));
+    c_path = git_repository_path(self->repo);
+    if (c_path == NULL)
+        Py_RETURN_NONE;
+
+    return to_path(c_path);
 }
 
 
@@ -391,17 +481,12 @@ Repository_workdir__get__(Repository *self, void *closure)
 int
 Repository_workdir__set__(Repository *self, PyObject *py_workdir)
 {
-    int err;
-    const char *workdir;
-    PyObject *tworkdir;
-
-    workdir = py_str_borrow_c_str(&tworkdir, py_workdir, NULL);
+    const char *workdir = pgit_borrow(py_workdir);
     if (workdir == NULL)
         return -1;
 
-    err = git_repository_set_workdir(self->repo, workdir, 0 /* update_gitlink */);
-    Py_DECREF(tworkdir);
-    if (err < 0) {
+    int err = git_repository_set_workdir(self->repo, workdir, 0 /* update_gitlink */);
+    if (err) {
         Error_set_str(err, workdir);
         return -1;
     }
@@ -481,6 +566,82 @@ Repository_merge_base(Repository *self, PyObject *args)
     return git_oid_to_python(&oid);
 }
 
+typedef int (*git_merge_base_xxx_t)(git_oid *out, git_repository *repo, size_t length, const git_oid input_array[]);
+
+static PyObject *
+merge_base_xxx(Repository *self, PyObject *args, git_merge_base_xxx_t git_merge_base_xxx)
+{
+    PyObject *py_result = NULL;
+    PyObject *py_commit_oid;
+    PyObject *py_commit_oids;
+    git_oid oid;
+    int commit_oid_count;
+    git_oid *commit_oids = NULL;
+    int i = 0;
+    int err;
+
+    if (!PyArg_ParseTuple(args, "O!", &PyList_Type, &py_commit_oids))
+        return NULL;
+
+    commit_oid_count = (int)PyList_Size(py_commit_oids);
+    commit_oids = malloc(commit_oid_count * sizeof(git_oid));
+    if (commit_oids == NULL) {
+        PyErr_SetNone(PyExc_MemoryError);
+        goto out;
+    }
+
+    for (; i < commit_oid_count; i++) {
+        py_commit_oid = PyList_GET_ITEM(py_commit_oids, i);
+        err = py_oid_to_git_oid_expand(self->repo, py_commit_oid, &commit_oids[i]);
+        if (err < 0)
+            goto out;
+    }
+
+    err = (*git_merge_base_xxx)(&oid, self->repo, commit_oid_count, (const git_oid*)commit_oids);
+
+    if (err == GIT_ENOTFOUND) {
+        Py_INCREF(Py_None);
+        py_result = Py_None;
+        goto out;
+    }
+
+    if (err < 0) {
+        py_result = Error_set(err);
+        goto out;
+    }
+
+    py_result = git_oid_to_python(&oid);
+
+out:
+    free(commit_oids);
+    return py_result;
+}
+
+
+PyDoc_STRVAR(Repository_merge_base_many__doc__,
+  "merge_base_many(oids) -> Oid\n"
+  "\n"
+  "Find as good common ancestors as possible for an n-way merge.\n"
+  "Returns None if there is no merge base between the commits");
+
+PyObject *
+Repository_merge_base_many(Repository *self, PyObject *args)
+{
+    return merge_base_xxx(self, args, &git_merge_base_many);
+}
+
+PyDoc_STRVAR(Repository_merge_base_octopus__doc__,
+  "merge_base_octopus(oids) -> Oid\n"
+  "\n"
+  "Find as good common ancestors as possible for an n-way octopus merge.\n"
+  "Returns None if there is no merge base between the commits");
+
+PyObject *
+Repository_merge_base_octopus(Repository *self, PyObject *args)
+{
+    return merge_base_xxx(self, args, &git_merge_base_octopus);
+}
+
 PyDoc_STRVAR(Repository_merge_analysis__doc__,
   "merge_analysis(their_head, our_ref='HEAD') -> (Integer, Integer)\n"
   "\n"
@@ -629,31 +790,33 @@ Repository_cherrypick(Repository *self, PyObject *py_oid)
 }
 
 PyDoc_STRVAR(Repository_walk__doc__,
-  "walk(oid[, sort_mode]) -> iterator\n"
-  "\n"
-  "Generator that traverses the history starting from the given commit.\n"
-  "The following types of sorting could be used to control traversing\n"
-  "direction:\n"
-  "\n"
-  "* GIT_SORT_NONE. This is the default sorting for new walkers.\n"
-  "  Sort the repository contents in no particular ordering\n"
-  "* GIT_SORT_TOPOLOGICAL. Sort the repository contents in topological order\n"
-  "  (parents before children); this sorting mode can be combined with\n"
-  "  time sorting.\n"
-  "* GIT_SORT_TIME. Sort the repository contents by commit time\n"
-  "* GIT_SORT_REVERSE. Iterate through the repository contents in reverse\n"
-  "  order; this sorting mode can be combined with any of the above.\n"
-  "\n"
-  "Example:\n"
-  "\n"
-  "  >>> from pygit2 import Repository\n"
-  "  >>> from pygit2 import GIT_SORT_TOPOLOGICAL, GIT_SORT_REVERSE\n"
-  "  >>> repo = Repository('.git')\n"
-  "  >>> for commit in repo.walk(repo.head.target, GIT_SORT_TOPOLOGICAL):\n"
-  "  ...    print(commit.message)\n"
-  "  >>> for commit in repo.walk(repo.head.target, GIT_SORT_TOPOLOGICAL | GIT_SORT_REVERSE):\n"
-  "  ...    print(commit.message)\n"
-  "  >>>\n");
+    "walk(oid[, sort_mode]) -> iterator\n"
+    "\n"
+    "Generator that traverses the history starting from the given commit.\n"
+    "The following types of sorting could be used to control traversing\n"
+    "direction:\n"
+    "\n"
+    "* GIT_SORT_NONE. Sort the output with the same default method from\n"
+    "  `git`: reverse chronological order. This is the default sorting for\n"
+    "  new walkers.\n"
+    "* GIT_SORT_TOPOLOGICAL. Sort the repository contents in topological order\n"
+    "  (no parents before all of its children are shown); this sorting mode\n"
+    "  can be combined with time sorting to produce `git`'s `--date-order``.\n"
+    "* GIT_SORT_TIME. Sort the repository contents by commit time; this sorting\n"
+    "  mode can be combined with topological sorting.\n"
+    "* GIT_SORT_REVERSE.  Iterate through the repository contents in reverse\n"
+    "  order; this sorting mode can be combined with any of the above.\n"
+    "\n"
+    "Example:\n"
+    "\n"
+    "  >>> from pygit2 import Repository\n"
+    "  >>> from pygit2 import GIT_SORT_TOPOLOGICAL, GIT_SORT_REVERSE\n"
+    "  >>> repo = Repository('.git')\n"
+    "  >>> for commit in repo.walk(repo.head.target, GIT_SORT_TOPOLOGICAL):\n"
+    "  ...    print(commit.message)\n"
+    "  >>> for commit in repo.walk(repo.head.target, GIT_SORT_TOPOLOGICAL | GIT_SORT_REVERSE):\n"
+    "  ...    print(commit.message)\n"
+    "  >>>\n");
 
 PyObject *
 Repository_walk(Repository *self, PyObject *args)
@@ -735,16 +898,17 @@ PyDoc_STRVAR(Repository_create_blob_fromworkdir__doc__,
     "is raised.");
 
 PyObject *
-Repository_create_blob_fromworkdir(Repository *self, PyObject *args)
+Repository_create_blob_fromworkdir(Repository *self, PyObject *py_path)
 {
-    git_oid oid;
-    const char* path;
-    int err;
-
-    if (!PyArg_ParseTuple(args, "s", &path))
+    PyObject *tvalue;
+    const char *path = pgit_borrow_encoding(py_path, Py_FileSystemDefaultEncoding,
+                                            Py_FileSystemDefaultEncodeErrors, &tvalue);
+    if (path == NULL)
         return NULL;
 
-    err = git_blob_create_fromworkdir(&oid, self->repo, path);
+    git_oid oid;
+    int err = git_blob_create_fromworkdir(&oid, self->repo, path);
+    Py_DECREF(tvalue);
     if (err < 0)
         return Error_set(err);
 
@@ -761,13 +925,18 @@ PyObject *
 Repository_create_blob_fromdisk(Repository *self, PyObject *args)
 {
     git_oid oid;
-    const char* path;
+    PyBytesObject *py_path = NULL;
+    const char* path = NULL;
     int err;
 
-    if (!PyArg_ParseTuple(args, "s", &path))
+    if (!PyArg_ParseTuple(args, "O&", PyUnicode_FSConverter, &py_path))
         return NULL;
 
+    if (py_path != NULL)
+      path = PyBytes_AS_STRING(py_path);
+
     err = git_blob_create_fromdisk(&oid, self->repo, path);
+    Py_XDECREF(py_path);
     if (err < 0)
         return Error_set(err);
 
@@ -866,16 +1035,13 @@ Repository_create_commit(Repository *self, PyObject *args)
     Signature *py_author, *py_committer;
     PyObject *py_oid, *py_message, *py_parents, *py_parent;
     PyObject *py_result = NULL;
-    PyObject *tmessage;
-    const char *message = NULL;
     char *update_ref = NULL;
     char *encoding = NULL;
     git_oid oid;
     git_tree *tree = NULL;
     int parent_count;
     git_commit **parents = NULL;
-    int err = 0, i = 0;
-    size_t len;
+    int i = 0;
 
     if (!PyArg_ParseTuple(args, "zO!O!OOO!|s",
                           &update_ref,
@@ -887,15 +1053,16 @@ Repository_create_commit(Repository *self, PyObject *args)
                           &encoding))
         return NULL;
 
-    len = py_oid_to_git_oid(py_oid, &oid);
+    size_t len = py_oid_to_git_oid(py_oid, &oid);
     if (len == 0)
         return NULL;
 
-    message = py_str_borrow_c_str(&tmessage, py_message, encoding);
+    PyObject *tmessage;
+    const char *message = pgit_borrow_encoding(py_message, encoding, NULL, &tmessage);
     if (message == NULL)
         return NULL;
 
-    err = git_tree_lookup_prefix(&tree, self->repo, &oid, len);
+    int err = git_tree_lookup_prefix(&tree, self->repo, &oid, len);
     if (err < 0) {
         Error_set(err);
         goto out;
@@ -1017,13 +1184,13 @@ Repository_create_branch(Repository *self, PyObject *args)
 }
 
 
-PyDoc_STRVAR(Repository_listall_references__doc__,
-  "listall_references() -> [str, ...]\n"
-  "\n"
-  "Return a list with all the references in the repository.");
+static PyObject *
+to_path_f(const char * x) {
+    return to_path(x);
+}
 
-PyObject *
-Repository_listall_references(Repository *self, PyObject *args)
+static PyObject *
+Repository_listall_references_impl(Repository *self, PyObject *args, PyObject *(*item_trans)(const char *))
 {
     git_strarray c_result;
     PyObject *py_result, *py_string;
@@ -1042,7 +1209,7 @@ Repository_listall_references(Repository *self, PyObject *args)
 
     /* Fill it */
     for (index=0; index < c_result.count; index++) {
-        py_string = to_path(c_result.strings[index]);
+        py_string = item_trans(c_result.strings[index]);
         if (py_string == NULL) {
             Py_CLEAR(py_result);
             goto out;
@@ -1055,6 +1222,28 @@ out:
     return py_result;
 }
 
+PyDoc_STRVAR(Repository_listall_references__doc__,
+  "listall_references() -> [str, ...]\n"
+  "\n"
+  "Return a list with all the references in the repository.");
+
+PyObject *
+Repository_listall_references(Repository *self, PyObject *args)
+{
+    return Repository_listall_references_impl(self, args, to_path_f);
+}
+
+PyDoc_STRVAR(Repository_raw_listall_references__doc__,
+  "raw_listall_references() -> [bytes, ...]\n"
+  "\n"
+  "Return a list with all the references in the repository.");
+
+PyObject *
+Repository_raw_listall_references(Repository *self, PyObject *args)
+{
+    return Repository_listall_references_impl(self, args, PyBytes_FromString);
+}
+
 
 PyDoc_STRVAR(Repository_listall_reference_objects__doc__,
   "listall_reference_objects() -> [Reference, ...]\n"
@@ -1106,19 +1295,8 @@ error:
 }
 
 
-PyDoc_STRVAR(Repository_listall_branches__doc__,
-  "listall_branches([flag]) -> [str, ...]\n"
-  "\n"
-  "Return a list with all the branches in the repository.\n"
-  "\n"
-  "The *flag* may be:\n"
-  "\n"
-  "- GIT_BRANCH_LOCAL - return all local branches (set by default)\n"
-  "- GIT_BRANCH_REMOTE - return all remote-tracking branches\n"
-  "- GIT_BRANCH_ALL - return local branches and remote-tracking branches");
-
-PyObject *
-Repository_listall_branches(Repository *self, PyObject *args)
+static PyObject *
+Repository_listall_branches_impl(Repository *self, PyObject *args, PyObject *(*item_trans)(const char *))
 {
     git_branch_t list_flags = GIT_BRANCH_LOCAL;
     git_branch_iterator *iter;
@@ -1139,7 +1317,7 @@ Repository_listall_branches(Repository *self, PyObject *args)
         return Error_set(err);
 
     while ((err = git_branch_next(&ref, &type, iter)) == 0) {
-        PyObject *py_branch_name = to_path(git_reference_shorthand(ref));
+        PyObject *py_branch_name = item_trans(git_reference_shorthand(ref));
         git_reference_free(ref);
 
         if (py_branch_name == NULL)
@@ -1169,6 +1347,40 @@ error:
     return NULL;
 }
 
+PyDoc_STRVAR(Repository_listall_branches__doc__,
+  "listall_branches([flag]) -> [str, ...]\n"
+  "\n"
+  "Return a list with all the branches in the repository.\n"
+  "\n"
+  "The *flag* may be:\n"
+  "\n"
+  "- GIT_BRANCH_LOCAL - return all local branches (set by default)\n"
+  "- GIT_BRANCH_REMOTE - return all remote-tracking branches\n"
+  "- GIT_BRANCH_ALL - return local branches and remote-tracking branches");
+
+PyObject *
+Repository_listall_branches(Repository *self, PyObject *args)
+{
+    return Repository_listall_branches_impl(self, args, to_path_f);
+}
+
+PyDoc_STRVAR(Repository_raw_listall_branches__doc__,
+  "raw_listall_branches([flag]) -> [bytes, ...]\n"
+  "\n"
+  "Return a list with all the branches in the repository.\n"
+  "\n"
+  "The *flag* may be:\n"
+  "\n"
+  "- GIT_BRANCH_LOCAL - return all local branches (set by default)\n"
+  "- GIT_BRANCH_REMOTE - return all remote-tracking branches\n"
+  "- GIT_BRANCH_ALL - return local branches and remote-tracking branches");
+
+PyObject *
+Repository_raw_listall_branches(Repository *self, PyObject *args)
+{
+    return Repository_listall_branches_impl(self, args, PyBytes_FromString);
+}
+
 PyDoc_STRVAR(Repository_listall_submodules__doc__,
   "listall_submodules() -> [str, ...]\n"
   "\n"
@@ -1179,23 +1391,25 @@ static int foreach_path_cb(git_submodule *submodule, const char *name, void *pay
     PyObject *list = (PyObject *)payload;
     PyObject *path = to_unicode(git_submodule_path(submodule), NULL, NULL);
 
-    return PyList_Append(list, path);
+    int err = PyList_Append(list, path);
+    Py_DECREF(path);
+    return err;
 }
 
 PyObject *
 Repository_listall_submodules(Repository *self, PyObject *args)
 {
-    int err;
-    PyObject *list;
-
-    list = PyList_New(0);
+    PyObject *list = PyList_New(0);
     if (list == NULL)
         return NULL;
 
-    err = git_submodule_foreach(self->repo, foreach_path_cb, list);
-    if (err != 0) {
+    int err = git_submodule_foreach(self->repo, foreach_path_cb, list);
+    if (err) {
         Py_DECREF(list);
-        return Py_None;
+        if (PyErr_Occurred())
+            return NULL;
+
+        return Error_set(err);
     }
 
     return list;
@@ -1220,16 +1434,14 @@ Repository_init_submodules(Repository* self, PyObject *args, PyObject *kwds)
     PyObject *list = Py_None;
     PyObject *oflag = Py_False;
     char *kwlist[] = {"submodules", "overwrite", NULL};
-    int err, fflag;
-    PyObject *iter, *subpath, *next;
+    int err;
     const char *c_subpath;
     git_submodule *submodule;
 
     if (!PyArg_ParseTupleAndKeywords(args, kwds, "|OO", kwlist, &list, &oflag))
         return NULL;
 
-    fflag = PyObject_IsTrue(oflag);
-
+    int fflag = PyObject_IsTrue(oflag);
     if (fflag != 0 && fflag != 1)
         fflag = 0;
 
@@ -1241,32 +1453,42 @@ Repository_init_submodules(Repository* self, PyObject *args, PyObject *kwds)
         Py_RETURN_NONE;
     }
 
-    iter = PyObject_GetIter(list);
+    PyObject *iter = PyObject_GetIter(list);
     if (!iter)
         return NULL;
 
+    PyObject *next = NULL;
     while (1) {
+        Py_XDECREF(next); // Decref from the previous iteration
         next = PyIter_Next(iter);
-        if (!next)
+        if (next == NULL) // List exhausted
             break;
 
-        c_subpath = py_str_borrow_c_str(&subpath, next, NULL);
+        c_subpath = pgit_borrow(next);
+        if (c_subpath == NULL)
+            goto error;
 
-        git_submodule_lookup(&submodule, self->repo, c_subpath);
-        Py_DECREF(subpath);
+        err = git_submodule_lookup(&submodule, self->repo, c_subpath);
         if (!submodule) {
-            PyErr_SetString(PyExc_KeyError,
-                "Submodule does not exist");
-            return NULL;
+            PyErr_SetString(PyExc_KeyError, "Submodule does not exist");
+            goto error;
         }
 
         err = git_submodule_init(submodule, fflag);
-        if (err != 0) {
-            return Error_set(err);
+        if (err) {
+            Error_set(err);
+            goto error;
         }
     }
 
+    // Success
+    Py_DECREF(iter);
     Py_RETURN_NONE;
+
+error:
+    Py_DECREF(iter);
+    Py_XDECREF(next);
+    return NULL;
 }
 
 PyDoc_STRVAR(Repository_lookup_reference__doc__,
@@ -1277,18 +1499,15 @@ PyDoc_STRVAR(Repository_lookup_reference__doc__,
 PyObject *
 Repository_lookup_reference(Repository *self, PyObject *py_name)
 {
-    git_reference *c_reference;
-    char *c_name;
-    int err;
-
     /* 1- Get the C name */
-    c_name = py_path_to_c_str(py_name);
+    char *c_name = pgit_encode_fsdefault(py_name);
     if (c_name == NULL)
         return NULL;
 
     /* 2- Lookup */
-    err = git_reference_lookup(&c_reference, self->repo, c_name);
-    if (err < 0) {
+    git_reference *c_reference;
+    int err = git_reference_lookup(&c_reference, self->repo, c_name);
+    if (err) {
         PyObject *err_obj = Error_set_str(err, c_name);
         free(c_name);
         return err_obj;
@@ -1307,18 +1526,15 @@ PyDoc_STRVAR(Repository_lookup_reference_dwim__doc__,
 PyObject *
 Repository_lookup_reference_dwim(Repository *self, PyObject *py_name)
 {
-    git_reference *c_reference;
-    char *c_name;
-    int err;
-
     /* 1- Get the C name */
-    c_name = py_path_to_c_str(py_name);
+    char *c_name = pgit_encode_fsdefault(py_name);
     if (c_name == NULL)
         return NULL;
 
     /* 2- Lookup */
-    err = git_reference_dwim(&c_reference, self->repo, c_name);
-    if (err < 0) {
+    git_reference *c_reference;
+    int err = git_reference_dwim(&c_reference, self->repo, c_name);
+    if (err) {
         PyObject *err_obj = Error_set_str(err, c_name);
         free(c_name);
         return err_obj;
@@ -1330,7 +1546,7 @@ Repository_lookup_reference_dwim(Repository *self, PyObject *py_name)
 }
 
 PyDoc_STRVAR(Repository_create_reference_direct__doc__,
-  "create_reference_direct(name, target, force)\n"
+  "create_reference_direct(name, target, force, message=None)\n"
   "\n"
   "Create a new reference \"name\" which points to an object.\n"
   "\n"
@@ -1342,6 +1558,9 @@ PyDoc_STRVAR(Repository_create_reference_direct__doc__,
   "    If True references will be overridden, otherwise (the default) an\n"
   "    exception is raised.\n"
   "\n"
+  "message\n"
+  "    Optional message to use for the reflog.\n"
+  "\n"
   "Examples::\n"
   "\n"
   "    repo.create_reference_direct('refs/heads/foo', repo.head.target, False)");
@@ -1355,15 +1574,18 @@ Repository_create_reference_direct(Repository *self,  PyObject *args,
     char *c_name;
     git_oid oid;
     int err, force;
+    const char *message = NULL;
+    char *keywords[] = {"name", "target", "force", "message", NULL};
 
-    if (!PyArg_ParseTuple(args, "sOi", &c_name, &py_obj, &force))
+    if (!PyArg_ParseTupleAndKeywords(args, kw, "sOi|z", keywords,
+                                     &c_name, &py_obj, &force, &message))
         return NULL;
 
     err = py_oid_to_git_oid_expand(self->repo, py_obj, &oid);
     if (err < 0)
         return NULL;
 
-    err = git_reference_create(&c_reference, self->repo, c_name, &oid, force, NULL);
+    err = git_reference_create(&c_reference, self->repo, c_name, &oid, force, message);
     if (err < 0)
         return Error_set(err);
 
@@ -1371,7 +1593,7 @@ Repository_create_reference_direct(Repository *self,  PyObject *args,
 }
 
 PyDoc_STRVAR(Repository_create_reference_symbolic__doc__,
-  "create_reference_symbolic(name, source, force)\n"
+  "create_reference_symbolic(name, source, force, message=None)\n"
   "\n"
   "Create a new reference \"name\" which points to another reference.\n"
   "\n"
@@ -1383,6 +1605,9 @@ PyDoc_STRVAR(Repository_create_reference_symbolic__doc__,
   "    If True references will be overridden, otherwise (the default) an\n"
   "    exception is raised.\n"
   "\n"
+  "message\n"
+  "    Optional message to use for the reflog.\n"
+  "\n"
   "Examples::\n"
   "\n"
   "    repo.create_reference_symbolic('refs/tags/foo', 'refs/heads/master', False)");
@@ -1394,12 +1619,15 @@ Repository_create_reference_symbolic(Repository *self,  PyObject *args,
     git_reference *c_reference;
     char *c_name, *c_target;
     int err, force;
+    const char *message = NULL;
+    char *keywords[] = {"name", "target", "force", "message", NULL};
 
-    if (!PyArg_ParseTuple(args, "ssi", &c_name, &c_target, &force))
+    if (!PyArg_ParseTupleAndKeywords(args, kw, "ssi|z", keywords,
+                                     &c_name, &c_target, &force, &message))
         return NULL;
 
     err = git_reference_symbolic_create(&c_reference, self->repo, c_name,
-                                        c_target, force, NULL);
+                                        c_target, force, message);
     if (err < 0)
         return Error_set(err);
 
@@ -1445,14 +1673,14 @@ Repository_status(Repository *self)
     size_t len, i;
     git_status_list *list;
 
-    dict = PyDict_New();
-    if (dict == NULL)
-        return NULL;
-
     err = git_status_list_new(&list, self->repo, NULL);
     if (err < 0)
         return Error_set(err);
 
+    dict = PyDict_New();
+    if (dict == NULL)
+        goto error;
+
     len = git_status_list_entrycount(list);
     for (i = 0; i < len; i++) {
         const git_status_entry *entry;
@@ -1496,20 +1724,19 @@ PyDoc_STRVAR(Repository_status_file__doc__,
 PyObject *
 Repository_status_file(Repository *self, PyObject *value)
 {
-    char *path;
-    unsigned int status;
-    int err;
-
-    path = py_path_to_c_str(value);
+    char *path = pgit_encode_fsdefault(value);
     if (!path)
         return NULL;
 
-    err = git_status_file(&status, self->repo, path);
-    if (err < 0) {
-        PyObject *err_obj =  Error_set_str(err, path);
+    unsigned int status;
+    int err = git_status_file(&status, self->repo, path);
+    if (err) {
+        PyObject *err_obj = Error_set_str(err, path);
         free(path);
         return err_obj;
     }
+    free(path);
+
     return PyLong_FromLong(status);
 }
 
@@ -1600,6 +1827,21 @@ Repository_odb__get__(Repository *self)
     return wrap_odb(odb);
 }
 
+PyDoc_STRVAR(Repository_refdb__doc__, "Return the reference database for this repository");
+
+PyObject *
+Repository_refdb__get__(Repository *self)
+{
+    git_refdb *refdb;
+    int err;
+
+    err = git_repository_refdb(&refdb, self->repo);
+    if (err < 0)
+        return Error_set(err);
+
+    return wrap_refdb(refdb);
+}
+
 PyDoc_STRVAR(Repository__pointer__doc__, "Get the repo's pointer. For internal use only.");
 PyObject *
 Repository__pointer__get__(Repository *self)
@@ -1613,26 +1855,26 @@ PyDoc_STRVAR(Repository_notes__doc__, "");
 PyObject *
 Repository_notes(Repository *self, PyObject *args)
 {
-    NoteIter *iter = NULL;
     char *ref = "refs/notes/commits";
-    int err = GIT_ERROR;
-
     if (!PyArg_ParseTuple(args, "|s", &ref))
         return NULL;
 
-    iter = PyObject_New(NoteIter, &NoteIterType);
-    if (iter != NULL) {
-        iter->repo = self;
-        iter->ref = ref;
+    NoteIter *iter = PyObject_New(NoteIter, &NoteIterType);
+    if (iter == NULL)
+        return NULL;
 
-        err = git_note_iterator_new(&iter->iter, self->repo, iter->ref);
-        if (err == GIT_OK) {
-            Py_INCREF(self);
-            return (PyObject*)iter;
-        }
+    Py_INCREF(self);
+    iter->repo = self;
+    iter->ref = ref;
+    iter->iter = NULL;
+
+    int err = git_note_iterator_new(&iter->iter, self->repo, iter->ref);
+    if (err != GIT_OK) {
+        Py_DECREF(iter);
+        return Error_set(err);
     }
 
-    return Error_set(err);
+    return (PyObject*)iter;
 }
 
 
@@ -1781,20 +2023,25 @@ PyObject *
 Repository_add_worktree(Repository *self, PyObject *args)
 {
     char *c_name;
-    char *c_path;
+    PyBytesObject *py_path = NULL;
+    char *c_path = NULL;
     Reference *py_reference = NULL;
     git_worktree *wt;
     git_worktree_add_options add_opts = GIT_WORKTREE_ADD_OPTIONS_INIT;
-    
+
     int err;
 
-    if (!PyArg_ParseTuple(args, "ss|O!", &c_name, &c_path, &ReferenceType, &py_reference))
+    if (!PyArg_ParseTuple(args, "sO&|O!", &c_name, PyUnicode_FSConverter, &py_path, &ReferenceType, &py_reference))
         return NULL;
 
+    if (py_path != NULL)
+        c_path = PyBytes_AS_STRING(py_path);
+
     if(py_reference != NULL)
         add_opts.ref = py_reference->reference;
-    
+
     err = git_worktree_add(&wt, self->repo, c_name, c_path, &add_opts);
+    Py_XDECREF(py_path);
     if (err < 0)
         return Error_set(err);
 
@@ -1860,31 +2107,171 @@ out:
 }
 
 PyDoc_STRVAR(Repository_apply__doc__,
-  "apply(id)\n"
+  "apply(diff, location=GIT_APPLY_LOCATION_WORKDIR)\n"
   "\n"
-  "Applies the given id into HEAD.\n"
+  "Applies the given Diff object to HEAD, writing the results into the\n"
+  "working directory, the index, or both.\n"
   "\n"
-  "Applies a diff into HEAD, writing the results into the\n"
-  "working directory.");
+  "Parameters:\n"
+  "\n"
+  "diff\n"
+  "    The Diff to apply.\n"
+  "\n"
+  "location\n"
+  "    The location to apply: GIT_APPLY_LOCATION_WORKDIR (default),\n"
+  "    GIT_APPLY_LOCATION_INDEX, or GIT_APPLY_LOCATION_BOTH.\n"
+  );
 
 PyObject *
-Repository_apply(Repository *self, PyObject *py_diff)
+Repository_apply(Repository *self, PyObject *args, PyObject *kwds)
 {
-    int err;
-    git_apply_location_t location = GIT_APPLY_LOCATION_WORKDIR;
+    Diff *py_diff;
+    int location = GIT_APPLY_LOCATION_WORKDIR;
     git_apply_options options = GIT_APPLY_OPTIONS_INIT;
 
-    err = git_apply(self->repo, ((Diff*)py_diff)->diff, location, &options);
+    char* keywords[] = {"diff", "location", NULL};
 
-    if (err < 0)
+    if (!PyArg_ParseTupleAndKeywords(args, kwds, "O!|i", keywords,
+                                     &DiffType, &py_diff,
+                                     &location))
+        return NULL;
+
+    int err = git_apply(self->repo, py_diff->diff, location, &options);
+    if (err != 0)
         return Error_set(err);
 
     Py_RETURN_NONE;
 }
 
+PyDoc_STRVAR(Repository_applies__doc__,
+  "applies(diff, location=GIT_APPLY_LOCATION_INDEX) -> bool\n"
+  "\n"
+  "Tests if the given patch will apply to HEAD, without writing it.\n"
+  "\n"
+  "Parameters:\n"
+  "\n"
+  "diff\n"
+  "    The Diff to apply.\n"
+  "\n"
+  "location\n"
+  "    The location to apply: GIT_APPLY_LOCATION_WORKDIR,\n"
+  "    GIT_APPLY_LOCATION_INDEX (default), or GIT_APPLY_LOCATION_BOTH.\n"
+  );
+
+PyObject *
+Repository_applies(Repository *self, PyObject *args, PyObject *kwds)
+{
+    Diff *py_diff;
+    int location = GIT_APPLY_LOCATION_INDEX;
+    git_apply_options options = GIT_APPLY_OPTIONS_INIT;
+    options.flags |= GIT_APPLY_CHECK;
+
+    char* keywords[] = {"diff", "location", NULL};
+
+    if (!PyArg_ParseTupleAndKeywords(args, kwds, "O!|i", keywords,
+                                     &DiffType, &py_diff,
+                                     &location))
+        return NULL;
+
+    int err = git_apply(self->repo, ((Diff*)py_diff)->diff, location, &options);
+    if (err != 0)
+        Py_RETURN_FALSE;
+
+    Py_RETURN_TRUE;
+}
+
+PyDoc_STRVAR(Repository_set_odb__doc__,
+  "set_odb(odb: Odb)\n"
+  "\n"
+  "Sets the object database for this repository.\n"
+  "This is a low-level function, most users won't need it.\n");
+
+PyObject *
+Repository_set_odb(Repository *self, Odb *odb)
+{
+    int err;
+    err = git_repository_set_odb(self->repo, odb->odb);
+    if (err < 0)
+        return Error_set(err);
+    Py_RETURN_NONE;
+}
+
+PyDoc_STRVAR(Repository_set_refdb__doc__,
+  "set_refdb(refdb: Refdb)\n"
+  "\n"
+  "Sets the reference database for this repository.\n"
+  "This is a low-level function, most users won't need it.\n");
+
+PyObject *
+Repository_set_refdb(Repository *self, Refdb *refdb)
+{
+    int err;
+    err = git_repository_set_refdb(self->repo, refdb->refdb);
+    if (err < 0)
+        return Error_set(err);
+    Py_RETURN_NONE;
+}
+
+static int foreach_stash_cb(size_t index, const char *message, const git_oid *stash_id, void *payload)
+{
+    int err;
+    Stash *py_stash;
+
+    py_stash = PyObject_New(Stash, &StashType);
+    if (py_stash == NULL)
+        return GIT_EUSER;
+
+    assert(message != NULL);
+    assert(stash_id != NULL);
+
+    py_stash->commit_id = git_oid_to_python(stash_id);
+    if (py_stash->commit_id == NULL)
+        return GIT_EUSER;
+
+    py_stash->message = strdup(message);
+    if (py_stash->message == NULL) {
+        PyErr_NoMemory();
+        return GIT_EUSER;
+    }
+
+    PyObject* list = (PyObject*) payload;
+    err = PyList_Append(list, (PyObject*) py_stash);
+    Py_DECREF(py_stash);
+    if (err < 0)
+        return GIT_EUSER;
+
+    return 0;
+}
+
+PyDoc_STRVAR(Repository_listall_stashes__doc__,
+  "listall_stashes() -> [Stash, ...]\n"
+  "\n"
+  "Return a list with all stashed commits in the repository.\n");
+
+PyObject *
+Repository_listall_stashes(Repository *self, PyObject *args)
+{
+    int err;
+
+    PyObject *list = PyList_New(0);
+    if (list == NULL)
+        return NULL;
+
+    err = git_stash_foreach(self->repo, foreach_stash_cb, (void*)list);
+
+    if (err == 0) {
+        return list;
+    } else {
+        Py_CLEAR(list);
+        if (PyErr_Occurred())
+            return NULL;
+        return Error_set(err);
+    }
+}
+
 PyMethodDef Repository_methods[] = {
     METHOD(Repository, create_blob, METH_VARARGS),
-    METHOD(Repository, create_blob_fromworkdir, METH_VARARGS),
+    METHOD(Repository, create_blob_fromworkdir, METH_O),
     METHOD(Repository, create_blob_fromdisk, METH_VARARGS),
     METHOD(Repository, create_blob_fromiobase, METH_O),
     METHOD(Repository, create_commit, METH_VARARGS),
@@ -1893,20 +2280,26 @@ PyMethodDef Repository_methods[] = {
     METHOD(Repository, walk, METH_VARARGS),
     METHOD(Repository, descendant_of, METH_VARARGS),
     METHOD(Repository, merge_base, METH_VARARGS),
+    METHOD(Repository, merge_base_many, METH_VARARGS),
+    METHOD(Repository, merge_base_octopus, METH_VARARGS),
     METHOD(Repository, merge_analysis, METH_VARARGS),
     METHOD(Repository, merge, METH_O),
     METHOD(Repository, cherrypick, METH_O),
-    METHOD(Repository, apply, METH_O),
-    METHOD(Repository, create_reference_direct, METH_VARARGS),
-    METHOD(Repository, create_reference_symbolic, METH_VARARGS),
+    METHOD(Repository, apply, METH_VARARGS | METH_KEYWORDS),
+    METHOD(Repository, applies, METH_VARARGS | METH_KEYWORDS),
+    METHOD(Repository, create_reference_direct, METH_VARARGS | METH_KEYWORDS),
+    METHOD(Repository, create_reference_symbolic, METH_VARARGS | METH_KEYWORDS),
     METHOD(Repository, compress_references, METH_NOARGS),
     METHOD(Repository, listall_references, METH_NOARGS),
+    METHOD(Repository, raw_listall_references, METH_NOARGS),
     METHOD(Repository, listall_reference_objects, METH_NOARGS),
     METHOD(Repository, listall_submodules, METH_NOARGS),
     METHOD(Repository, init_submodules, METH_VARARGS | METH_KEYWORDS),
     METHOD(Repository, lookup_reference, METH_O),
     METHOD(Repository, lookup_reference_dwim, METH_O),
     METHOD(Repository, revparse_single, METH_O),
+    METHOD(Repository, revparse_ext, METH_O),
+    METHOD(Repository, revparse, METH_O),
     METHOD(Repository, status, METH_NOARGS),
     METHOD(Repository, status_file, METH_O),
     METHOD(Repository, notes, METH_VARARGS),
@@ -1916,6 +2309,7 @@ PyMethodDef Repository_methods[] = {
     METHOD(Repository, lookup_branch, METH_VARARGS),
     METHOD(Repository, path_is_ignored, METH_VARARGS),
     METHOD(Repository, listall_branches, METH_VARARGS),
+    METHOD(Repository, raw_listall_branches, METH_VARARGS),
     METHOD(Repository, create_branch, METH_VARARGS),
     METHOD(Repository, reset, METH_VARARGS),
     METHOD(Repository, free, METH_NOARGS),
@@ -1925,6 +2319,9 @@ PyMethodDef Repository_methods[] = {
     METHOD(Repository, list_worktrees, METH_VARARGS),
     METHOD(Repository, _from_c, METH_VARARGS),
     METHOD(Repository, _disown, METH_NOARGS),
+    METHOD(Repository, set_odb, METH_O),
+    METHOD(Repository, set_refdb, METH_O),
+    METHOD(Repository, listall_stashes, METH_NOARGS),
     {NULL}
 };
 
@@ -1935,9 +2332,11 @@ PyGetSetDef Repository_getseters[] = {
     GETTER(Repository, head_is_unborn),
     GETTER(Repository, is_empty),
     GETTER(Repository, is_bare),
+    GETTER(Repository, is_shallow),
     GETSET(Repository, workdir),
     GETTER(Repository, default_signature),
     GETTER(Repository, odb),
+    GETTER(Repository, refdb),
     GETTER(Repository, _pointer),
     {NULL}
 };
