@@ -1,5 +1,5 @@
 /*
- * Copyright 2010-2019 The pygit2 contributors
+ * Copyright 2010-2021 The pygit2 contributors
  *
  * This file is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License, version 2,
@@ -32,8 +32,8 @@
 #include <Python.h>
 #include <git2.h>
 
-#if !(LIBGIT2_VER_MAJOR == 0 && LIBGIT2_VER_MINOR == 28)
-#error You need a compatible libgit2 version (v0.28.x)
+#if !(LIBGIT2_VER_MAJOR == 1 && LIBGIT2_VER_MINOR == 3)
+#error You need a compatible libgit2 version (1.3.x)
 #endif
 
 /*
@@ -74,6 +74,20 @@ typedef struct {
     OdbBackend super;
 } OdbBackendLoose;
 
+typedef struct {
+    PyObject_HEAD
+    git_refdb *refdb;
+} Refdb;
+
+typedef struct {
+    PyObject_HEAD
+    git_refdb_backend *refdb_backend;
+} RefdbBackend;
+
+typedef struct {
+    RefdbBackend super;
+} RefdbFsBackend;
+
 
 #define SIMPLE_TYPE(_name, _ptr_type, _ptr_name) \
         typedef struct {\
@@ -192,8 +206,6 @@ typedef struct {
 
 
 /* git_index */
-SIMPLE_TYPE(Index, git_index, index)
-
 typedef struct {
     PyObject_HEAD
     git_index_entry entry;
@@ -222,6 +234,13 @@ typedef struct {
     size_t size;
 } RefLogIter;
 
+/* git_revspec */
+typedef struct {
+    PyObject_HEAD
+    PyObject *from;
+    PyObject *to;
+    unsigned int flags;
+} RevSpec;
 
 /* git_signature */
 typedef struct {
@@ -237,4 +256,10 @@ typedef struct {
     git_mailmap *mailmap;
 } Mailmap;
 
+typedef struct {
+    PyObject_HEAD
+    PyObject *commit_id;
+    char *message;
+} Stash;
+
 #endif
